"""Factories for the ``hero_slider`` app."""
import factory

from filer.models.imagemodels import Image

from hero_slider.models import SliderItem, SliderItemTitle
from hero_slider.tests.test_app.models import DummyModel


class DummyModelFactory(factory.Factory):
    """Factory for ``DummyModel`` objects."""
    FACTORY_FOR = DummyModel

    name = 'A name'


class FilerImageFactory(factory.Factory):
    """Factory for ``Image`` objects of the django-filer app."""
    FACTORY_FOR = Image

    owner = None
    original_filename = 'foobar.jpg'
    file = None


class SliderItemFactory(factory.Factory):
    """Factory for ``SliderItem`` objects."""
    FACTORY_FOR = SliderItem

    image = factory.SubFactory(FilerImageFactory)
    content_object = factory.SubFactory(DummyModelFactory)


class SliderItemTitleFactoryBase(factory.Factory):
    """Base class for ``SliderItemTitle`` factories."""
    FACTORY_FOR = SliderItemTitle

    slider_item = factory.SubFactory(SliderItemFactory)


class SliderItemTitleENFactory(SliderItemTitleFactoryBase):
    """Factory for english ``SliderItemTitle`` objects."""
    title = 'A title'
    description = 'A description'
    language = 'en'
    is_published = True


class SliderItemTitleDEFactory(SliderItemTitleFactoryBase):
    """Factory for german ``SliderItemTitle`` objects."""
    title = 'Ein Titel'
    description = 'Eine Beschreibung'
    language = 'de'
    is_published = True



"""Admin classes for the ``hero_slider`` app."""
from django.contrib import admin
from django.utils.translation import get_language
from django.utils.translation import ugettext_lazy as _

from django_libs.admin import MultilingualPublishMixin
from simple_translation.admin import TranslationAdmin
from simple_translation.utils import get_preferred_translation_from_lang

from hero_slider.models import SliderItem


class SliderItemAdmin(MultilingualPublishMixin, TranslationAdmin):
    """Admin for the ``SliderItem`` model."""
    list_display = ['title', 'position', 'languages', 'is_published']

    def title(self, obj):
        lang = get_language()
        return get_preferred_translation_from_lang(obj, lang).title
    title.short_description = _('Title')


admin.site.register(SliderItem, SliderItemAdmin)



"""Tests for the templatetags of the ``hero_slider`` app."""
from django.template import RequestContext
from django.test import TestCase, RequestFactory

from hero_slider.templatetags.hero_slider_tags import render_hero_slider
from hero_slider.tests.factories import (
    SliderItemTitleDEFactory,
    SliderItemTitleENFactory,
)


class RenderHeroSliderTestCase(TestCase):
    """Tests for the ``render_hero_slider`` templatetag."""
    longMessage = True

    def setUp(self):
        self.de_title = SliderItemTitleDEFactory()
        SliderItemTitleENFactory(
            slider_item=self.de_title.slider_item, is_published=False)
        self.en_title = SliderItemTitleENFactory()
        SliderItemTitleDEFactory(
            slider_item=self.en_title.slider_item, is_published=False)

    def test_tag(self):
        req = RequestFactory().get('/')
        req.LANGUAGE_CODE = 'en'
        context = RequestContext(req)
        result = render_hero_slider(context)

        self.assertEqual(len(result['slider_items']), 1, msg=(
            'When set to English, it should return one item.'))
        self.assertEqual(
            result['slider_items'][0], self.en_title.slider_item, msg=(
                'When set to English, it should return the english item.'))

        req.LANGUAGE_CODE = 'de'
        context = RequestContext(req)
        result = render_hero_slider(context)
        self.assertEqual(len(result['slider_items']), 1, msg=(
            'When set to German, it should return one item.'))
        self.assertEqual(
            result['slider_items'][0], self.de_title.slider_item, msg=(
                'When set to English, it should return the german item.'))

        req = RequestFactory().get('/')
        context = RequestContext(req)
        result = render_hero_slider(context)
        self.assertEqual(len(result['slider_items']), 0, msg=(
            'When no language is set, it should return no items.'))



"""Tests for the models of the ``hero_slider`` app."""
from mock import Mock

from django.test import TestCase

from ..models import SliderItem
from .factories import (
    SliderItemFactory,
    SliderItemTitleDEFactory,
    SliderItemTitleENFactory,
)


class SliderItemTestCase(TestCase):
    """Tests for the ``SliderItem`` model."""
    longMessage = True

    def test_model(self):
        instance = SliderItemFactory()
        self.assertTrue(instance.pk, msg=(
            'Should be able to instantiate and save the model.'))

    def test_get_trans(self):
        item_trans = SliderItemTitleENFactory()
        self.assertEqual(item_trans.slider_item.get_trans(), item_trans, msg=(
            'Should return the translation object for the current language'))


class SliderItemManagerTestCase(TestCase):
    """Tests for the ``SliderItemManager`` model manager."""
    longMessage = True

    def setUp(self):
        self.en_title = SliderItemTitleENFactory(is_published=False)
        SliderItemTitleDEFactory(slider_item=self.en_title.slider_item)

        self.de_title = SliderItemTitleDEFactory(is_published=False)
        SliderItemTitleENFactory(slider_item=self.de_title.slider_item)

    def test_manager(self):
        """Test if the ``SliderItemManager`` retrieves the correct objects."""
        request = Mock(LANGUAGE_CODE='de')
        self.assertEqual(
            SliderItem.objects.published(request).count(), 1, msg=(
                'In German, there should be one published slider item.'))

        request = Mock(LANGUAGE_CODE='en')
        self.assertEqual(
            SliderItem.objects.published(request).count(), 1, msg=(
                'In English, there should be one published slider item.'))

        request = Mock(LANGUAGE_CODE=None)
        self.assertEqual(
            SliderItem.objects.published(request).count(), 0, msg=(
                'If no language is set, there should be no slider items.'))


class SliderItemTitleTestCase(TestCase):
    """Tests for the ``SliderItemTitle`` model."""
    longMessage = True

    def test_model(self):
        instance = SliderItemTitleENFactory()
        self.assertTrue(instance.pk, msg=(
            'Should be able to instantiate and save the model.'))



"""Tests for the views of the ``hero_slider`` app."""
from django.contrib.contenttypes.models import ContentType
from django.http import Http404
from django.test import RequestFactory, TestCase

from hero_slider.views import GetCTypeDetails


class GetCTypeDetailsTestCase(TestCase):
    """Tests for the ``GetCTypeDetails`` view class."""
    def test_view(self):
        req = RequestFactory().get('/')
        req.GET = {'pk': 1, }
        resp = GetCTypeDetails.as_view()(req)
        ctype = ContentType.objects.get(pk=1)
        self.assertTrue(ctype.app_label in resp.content, msg=(
            'Should return a JSON string containing app_label and model'))

    def test_bad_pk(self):
        req = RequestFactory().get('/')
        req.GET = {'pk': 999, }
        self.assertRaises(Http404, GetCTypeDetails.as_view(), req)



"""Settings that need to be set in order to run the tests."""
import os

DEBUG = True

SITE_ID = 1

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
    }
}

LANGUAGE_CODE = 'en'

LANGUAGES = (
    ('en', 'English'),
    ('de', 'German'),
)

ROOT_URLCONF = 'hero_slider.tests.urls'

STATIC_URL = '/static/'
MEDIA_URL = '/media/'

STATIC_ROOT = os.path.join(__file__, '../../../static/')
MEDIA_ROOT = os.path.join(__file__, '../../../media/')

STATICFILES_DIRS = (
    os.path.join(__file__, 'test_static'),
)

TEMPLATE_DIRS = (
    os.path.join(os.path.dirname(__file__), '../templates'),
)

COVERAGE_REPORT_HTML_OUTPUT_DIR = os.path.join(
    os.path.dirname(__file__), 'coverage')

COVERAGE_MODULE_EXCLUDES = [
    'tests$', 'settings$', 'urls$', 'locale$',
    'migrations', 'fixtures', 'admin$', 'django_extensions',
]

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.core.context_processors.request',
)

THUMBNAIL_PROCESSORS = (
    'easy_thumbnails.processors.colorspace',
    'easy_thumbnails.processors.autocrop',
    'filer.thumbnail_processors.scale_and_crop_with_subject_location',
    'easy_thumbnails.processors.filters',
)

EXTERNAL_APPS = [
    'django.contrib.admin',
    'django.contrib.admindocs',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.messages',
    'django.contrib.sessions',
    'django.contrib.staticfiles',
    'django.contrib.sitemaps',
    'django.contrib.sites',
    'django_nose',
    'filer',
    'easy_thumbnails',
]

INTERNAL_APPS = [
    'hero_slider.tests.test_app',
    'hero_slider',
]

INSTALLED_APPS = EXTERNAL_APPS + INTERNAL_APPS

COVERAGE_MODULE_EXCLUDES += EXTERNAL_APPS



"""URLs for the ``hero_slider`` app."""
from django.conf.urls.defaults import patterns, url

from hero_slider.views import GetCTypeDetails


urlpatterns = patterns(
    '',
    url(r'ctype/$', GetCTypeDetails.as_view(), name='get_ctype_details'),
)



"""Registering translated models for the ``hero_slider`` app."""
from simple_translation.translation_pool import translation_pool

from hero_slider.models import (
    SliderItem,
    SliderItemTitle,
)


translation_pool.register_translation(SliderItem, SliderItemTitle)



"""Models for the ``hero_slider`` app."""
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import get_language
from django.utils.translation import ugettext_lazy as _

from simple_translation.utils import get_preferred_translation_from_lang


from filer.fields.file import FilerFileField


class SliderItemManager(models.Manager):
    """Custom manager for the ``SliderItem`` model."""
    def published(self, request):
        """
        Returns the published slider items in the current language.

        :param request: A Request instance.

        """
        language = getattr(request, 'LANGUAGE_CODE', None)
        if not language:
            return self.model.objects.none()
        qs = self.get_query_set()
        qs = qs.filter(
            slideritemtitle__is_published=True,
            slideritemtitle__language=language,
        )
        return qs


class SliderItem(models.Model):
    """
    Resembles an item that should be shown on the front page in a slider.

    For translateable fields see the ``SliderItemTitle`` model.

    :image: A filer file. This will be the image shown in the slider.
    :position: Can be set in order to control the ordering of the slider items.
    :content_type: The contenttype of the object this slider item links to.
    :object_id: The PK of the object this slider item links to.
    :external_url: URL to link to items without the need of a gfk.

    """
    image = FilerFileField(
        verbose_name=_('Image'),
    )

    position = models.PositiveIntegerField(
        verbose_name=_('Position'),
        null=True, blank=True,
    )

    # Generic foreign key
    content_type = models.ForeignKey(ContentType, blank=True, null=True)
    object_id = models.PositiveIntegerField(blank=True, null=True)
    content_object = generic.GenericForeignKey('content_type', 'object_id')

    external_url = models.CharField(
        verbose_name=_('External URL'),
        max_length=512,
        blank=True,
    )

    objects = SliderItemManager()

    def get_trans(self):
        """Returns the translation object for this slider item."""
        lang = get_language()
        return get_preferred_translation_from_lang(self, lang)

    def get_item_url(self):
        """Returns the url of the connected item."""
        try:
            return self.content_object.get_absolute_url()
        except AttributeError:
            return self.external_url


class SliderItemTitle(models.Model):
    """
    Translateable fields of the ``SliderItem`` model.

    :title: The title of this slider item.
    :description: A short description of this slider item.
    :is_published: If True, this will not show up.

    """
    title = models.CharField(
        max_length=256,
        verbose_name=_('Title'),
        blank=True,
    )

    description = models.CharField(
        max_length=512,
        verbose_name=_('Description'),
        blank=True,
    )

    is_published = models.BooleanField(
        verbose_name=_('Is published'),
        default=False,
    )

    # Needed by simple-translation
    slider_item = models.ForeignKey(SliderItem, verbose_name=_('Slider item'))
    language = models.CharField(max_length=2, verbose_name=_('Language'))
